/* 
 * CSE142, 99 Winter
 * Homework 5
 * Valentine's Day
 *
 * Author:     Alex Glover
 * Student ID: 9824472
 * Section:    AA
 *	
 * Greeting Card Designer (Ultimate version)
 *
 *  The GP142 graphics window shows some buttons and the inside or
 *  outside (cover) of a greeting card. The idea is that you can
 *  design what you want to card to look like by adding and moving
 *  objects: 
 *    messages, message color and position
 *    shapes,   shape color and position
 *
 *   To place a message or a greeting, you click on an appropriate
 *   button, and then you can click on the desired position in the
 *   card.
 *
 */

#include "gp142.h"
#include <math.h>
#include <time.h>

#define TRUE    		1
#define FALSE   		0

/*
 * The state, or status you are in determines what happens when you
 * click the card:
 * ADD SHAPE     -- you click the card, and a new shape is placed
 *                  there. You get into this mode by clicking the add
 *                  shape button. After doing an ADD, you are put into
 *                  SELECTED mode.
 * ADD MESSAGE   -- you click the card, and a new message is placed
 *                  there. You get into this mode by clicking the add
 *                  message button. After doing an ADD, you are put
 *                  into SELECTED mode.
 * SELECTED      -- An object is selected. In this mode, the type 
 *                  rotation applies to the current object.
 * SELECT OBJECT -- if you click near the center of an object on the
 *                  card, that object becomes the selected object and
 *                  you are then put into SELECTED mode.
 * NO SELECT     -- you are put into this mode if you attempt to
 *                  select an object when there are none on the card.
 *                  Clicking on the card puts you into ADD SHAPE mode.
 * DELETE OBJECT -- if you click on the object currently selected, that
 *                  object is removed from the card. if you click on
 *                  another object, it is selected but not removed. You
 *                  get into this mode by clicking the delete object
 *                  button. After doing a DELETE, you are put into
 *                  SELECTED mode for the most recently added object.
 * NO DELETE     -- you are put into this mode if you attempt to
 *                  delete an object when there are none on the card.
 *                  Clicking on the card puts you into ADD SHAPE mode.
 * MAX OBJECTS   -- you are put in this mode when you have placed the
 *                  maximum number of objects on the card and attempt
 *                  to add another.
 * SEND CARD     -- this mode is enabled when you click on the send
 *                  card button. In this mode, button clicks are no
 *                  longer recognized and the send animation begins.
 *                  This animation is supposed to take place last, so
 *                  there is no way to exit this mode except by exiting
 *                  the program completely.
 */
#define STATUS_ADD_SHAPE		1
#define STATUS_ADD_MESSAGE		2
#define STATUS_SELECTED			3
#define STATUS_SELECT_OBJECT	4
#define STATUS_NO_SELECT		5
#define STATUS_DELETE_OBJECT	6
#define STATUS_NO_DELETE		7
#define STATUS_MAX_OBJECTS		8
#define STATUS_SEND_CARD		9

/* Configure the appearance of the buttons on the menu bar 
 * by setting the number of pixels of button height and width
 */
#define BUTTON_HEIGHT  	36
#define BUTTON_WIDTH 	100
#define SUB_BUTTON_HEIGHT	18
#define SUB_BUTTON_WIDTH	50

/* change this if you add or delete buttons*/
#define NUM_BUTTONS 8
#define SELECTED_COLOR  RED
/* These are the different buttons */
#define NO_BUTTON				-1
#define CHANGE_CARD_DESIGN		0
#define CHANGE_BACKGROUND_COLOR	1
#define CHANGE_OBJECT_COLOR		2
#define CHANGE_OBJECT_NUM		3
#define ADD_OBJECT				4
#define DELETE_OBJECT			5
#define SELECT_OBJECT			6
#define SEND_CARD				7

#define NUM_SUB_BUTTONS 2
/* These are the pairs of subordinate buttons */
#define NO_SUB_BUTTON	-1
#define NEXT			0
#define PREVIOUS		1
#define MESSAGE			0
#define SHAPE			1

/* Position of the card in the window */
#define PAGE_LLX (-GP142_XMAX + BUTTON_WIDTH + 60)
#define PAGE_LLY (-GP142_YMAX + 100)
#define PAGE_WIDTH (GP142_XMAX - PAGE_LLX - 80)
#define PAGE_HEIGHT (GP142_YMAX - PAGE_LLY - 50)

/* For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.
 */
#define FILL 0
#define NOFILL 1

/* options for DrawBevelRect function */
#define RAISED		0
#define RECESSED	1

/* Change this when adding new card designs. */
#define MAX_DESIGNS 5
#define FIRST_DESIGN_NUM 0
/* These are the different kinds of designs. */
#define NO_DESIGN			0
#define DESIGN_SPRING		1
#define DESIGN_CALENDAR		2
#define DESIGN_HEART		3
#define DESIGN_BORDER		4

/* Types of objects supported. */
#define OBJECT_NONE 0
#define OBJECT_SHAPE 1
#define OBJECT_MESSAGE 2
/* Maximum number of objects that can be placed on the card. */
#define MAX_OBJECTS	20

/* Change this when adding new shapes. */
#define MAX_SHAPES 6
#define FIRST_SHAPE_NUM 0
/* These are the different kinds of shapes. */
#define SHAPE_HEART		0
#define SHAPE_STAR		1
#define SHAPE_CIRCLE	2
#define SHAPE_DIAMOND	3
#define SHAPE_OVAL		4
#define SHAPE_FLOWER	5

/* Change this when adding new messages. */
#define MAX_MESSAGES 5
#define FIRST_MESSAGE_NUM 0
/* These are the different kinds of messages. */
#define MESSAGE_1	0
#define MESSAGE_2	1
#define MESSAGE_3	2
#define MESSAGE_4	3
#define MESSAGE_5	4

/*Possible places for mouse clicks*/
#define CLICK_BUTTON 1  /* You clicked on SOME button */
#define CLICK_CARD 2    /* You clicked in the card area */
#define CLICK_OUTSIDE 3	/* You clicked outside the card area,
						   not on a button*/	


/*******************  FUNCTION PROTOTYPES **************************/

/* Drawing functions */
void DrawBlankScreen (void);
void DrawTitle (void);
void DrawButtons(int last_button, int last_sub_button);
void DrawSubButtons(int button_num, int last_sub_button);
void DrawBevelRect(int x1, int y1, int x2, int y2, int status);
void DrawCardArea (int *time, int card_design, int background_color,
				   int status, int object_count,
				   const int object_type[], const int object_num[],
				   const int object_color[],
				   const int object_x[], const int object_y[]);
void DrawFlapUp(int time);
void DrawFlapClosing(int time);
void DrawFlapDown(int time);
void DrawEnvelope(int time);
void DrawCard (int card_design, int background_color, int status,
			   int object_count, const int object_type[],
			   const int object_num[], const int object_color[],
			   const int object_x[], const int object_y[]);
void DrawSpringDesign (void);
void DrawCalendarDesign (void);
void DrawHeartDesign (void);
void DrawBorderDesign (void);
void DrawShape(int color, int shape_type, int bottom_x, int bottom_y);
void DrawHeart (int x, int y, int color);
void DrawStar (int x, int y, int color);
void DrawCircle (int x, int y, int color);
void DrawDiamond (int x, int y, int color);
void DrawOval (int x, int y, int color);
void DrawFlower (int x, int y, int color);
void DrawMessage(int msg_type, int color, int bottom_x, int bottom_y);

/* Mouse click handling functions */
void HandleMouseClick(int mouse_x, int mouse_y, int *action_time,
					  int *card_design, int *background_color,
					  int *last_button, int *last_sub_button,
					  int *status, int *current_object,
					  int *object_count, int object_type[],
					  int object_num[], int object_color[],
					  int object_x[], int object_y[]);
int ClassifyMouseClick(int mouse_x, int mouse_y);
void HandleButton(int x, int y, int *action_time, int *card_design,
				  int *background_color, int *last_button,
				  int *last_sub_button, int *status,
				  int current_object, int object_count,
				  const int object_type[], int object_color[],
				  int object_num[]);
void HandleObject(int mouse_x, int mouse_y, int *status,
				  int *last_button, int *last_sub_button,
				  int *current_object, int *object_count,
				  int object_type[], int object_num[],
				  int object_color[], int object_x[], int object_y[]);

/* Perform button functions. */
void ChangeCardDesign(int *card_design, int direction);
void ChangeBackgroundColor(int *background_color, int direction);
void ChangeObjectColor(int *object_color, int direction);
void ChangeObjectNum(int *object_num, int object_type, int direction);

/* Perform object functions. */
void AddShape(int x, int y, int *status, int *current_object,
			  int *object_count, int *object_type, int *object_num,
			  int *object_color, int *object_x, int *object_y);
void AddMessage(int x, int y, int *status, int *current_object,
			  int *object_count, int *object_type, int *object_num,
			  int *object_color, int *object_x, int *object_y);
void ObjectMove(int x, int y, int *object_x, int *object_y);
void ObjectSelect(int x, int y, int *status, int *current_object,
				  int object_count,
				  const int object_x[], const int object_y[]);
void ObjectDelete(int x, int y, int *status, int *current_object,
				  int *object_count, int object_type[],
				  int object_num[], int object_color[],
				  int object_x[], int object_y[]);


/**********************************************************************

     MAIN

**********************************************************************/

/* main() has two chief purposes: to contain the important variables
 * and initialize them, and to contain the "event loop," which
 * constantly looks to see if an event has happened, and then processes
 * the event by calling an appropriate function. An example of an event
 * is a mouse click.
 */
int main(void)
{
	int quit = FALSE;              /* becomes TRUE to exit program */
	int mouse_x, mouse_y;          /* coordinates of a mouse click */
	char key_hit;                  /* which key was hit            */
	
	
	/*variables and their default values*/
	int card_design = NO_DESIGN;
	int background_color = YELLOW;
	int status = STATUS_ADD_SHAPE;
	
	/* Each object is represented in a set of parallel arrays,
	 * containing a number telling what kind of object it is, a color,
	 * and an x and y coordinate.
	 */
	int object_count = 0;			/* Number of objects on the page */
	int object_type[MAX_OBJECTS];	/* Either a shape or a message   */
	int object_num[MAX_OBJECTS];	/* The kind of shape or message  */
	int object_color[MAX_OBJECTS];	/* Color of object               */
	int object_x[MAX_OBJECTS];		/* Position of object            */
	int object_y[MAX_OBJECTS];
	
	/* These keep track of the last button clicked. They are used to
	 * highlight the last button the user clicked, and display
	 * subordinate buttons if appropriate. */
	int last_button = NO_BUTTON;
	int last_sub_button = NO_SUB_BUTTON;

	int current_object = -1;		/* Keep track of the object
									   currently selected */

	int action_time = -1;			/* Keep track of time for
									   animations */

	int event;	/* the most recent GP142 event */
	int i;		/* loop counter */
	
	time_t t;	/* used to initialize the random number generator */
	

	/*
	 * INITIALIZATIONS
	 */
	srand (time(&t));	    /* Seed the random number generator. */

	/* There are no objects when the program is first started. */
	for (i = 0; i < MAX_OBJECTS; i++) {
		object_type[i] = OBJECT_NONE;
	}

	/* GP142 initializations */
	GP142_open();           /* Open, init the GP142 Graphics Window */
	GP142_show_text(0);     /* Hide the text window					*/
	GP142_logging(LOG_OFF); /* Prevent logging						*/
	
	GP142_clear();	        /* Clear the window & paint it white.	*/
	GP142_animate(ANI_RUN); /* Enable periodic events				*/
		
	
	/*
	 * The main event loop:
	 * --- ---- ----- -----
	 * wait for the next user action, such as a mouse click or keyboard
	 * keypress, or a periodic update event (allowing us to display the
	 * next frame of an animation -- not used directly in this program)
	 * and call the appropriate function to handle it.  Repeat this
	 * until the user signals "quit".
	 */
	while (!quit) {
		
		/*start by redrawing everything */
		DrawBlankScreen ();
		DrawTitle ();
		DrawCardArea (&action_time, card_design, background_color,
					  status, object_count, object_type, object_num,
					  object_color, object_x, object_y);
		DrawButtons(last_button, last_sub_button);
		
		/*Wait for something to happen */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_hit);
		
		/* Something just happened.  Figure out what it was, and react 
		   appropriately. */
		switch (event) {
		case GP142_QUIT:
			quit = TRUE;	/* set flag to terminate loop */
			break;
			
		case GP142_KBD:		/* A key was pressed */
			/*not needed for this program. Use this if user will need
			  to hit a key. Note that scanf CANNOT be used with GP142.
			  If you use this, the key character value is in variable
			  "key_hit"*/
			break;
			
		case GP142_MOUSE:
			/* A mouse click has occurred. Call the mouse handler to 
			   decide where the click was and react as needed. */
			HandleMouseClick(mouse_x, mouse_y, &action_time,
							 &card_design, &background_color,
							 &last_button, &last_sub_button, &status,
							 &current_object, &object_count,
							 object_type, object_num, object_color,
							 object_x, object_y);
			break;
			
		case GP142_PERIODIC:
			/*An event has occurred simply because some time has
			  passed. (not currently used in the program) */		
			break;

		default:
			
			break;
			
		}  /*end switch */
		
	}  /* end event loop */
	
	
	/*
	 * WRAP-UP
	 */
	GP142_close();         /* clean up and close graphics window   */
	
	return 0;
}  /*end main */

/**********************************************************************

     FUNCTIONS

**********************************************************************/
/* 
 * For a blank screen just draw a white rectangle that covers the whole
 * GP142 window.
 */
void DrawBlankScreen (void) {
	int font_size = 14;

	GP142_rectangleXY (WHITE,
					-GP142_XMAX, -GP142_YMAX,
					GP142_XMAX, GP142_YMAX,
					FILL);

} /*end DrawBlankScreen */


/*
 * Draw a title box in the window, showing the name of the program and
 * related information.
 */
#define TITLE_WIDTH		125
#define TITLE_HEIGHT	200
void DrawTitle (void) {

	/* Draw beveled background */
	DrawBevelRect(-GP142_XMAX + 4,
  				  -GP142_YMAX + TITLE_HEIGHT,
				  -GP142_XMAX + TITLE_WIDTH,
				  -GP142_YMAX + 3,
				  RAISED);

	/* Print program title */
	GP142_printfXY(BLACK, -GP142_XMAX + 12, -GP142_YMAX + 150, 30,
				"ALEX'S");
	GP142_printfXY(RED, -GP142_XMAX + 10, -GP142_YMAX + 120, 20,
				"VALENTINE");
	GP142_printfXY(BLACK, -GP142_XMAX + 22, -GP142_YMAX + 80, 30,
				"CARD\nSHOP");
	GP142_printfXY(RED, -GP142_XMAX + 17, -GP142_YMAX + 30, 10,
				"Obscure Sect Edition");
	
}


/* 
 * Draws the buttons 
 */
void DrawButtons(int last_button, int last_sub_button)
{
/* To add buttons, change the #defined constant NUM_BUTTONS and add
 * messages to this array.
 */
	char b_labels [NUM_BUTTONS][25] = {
			 "Card design",
		"Background color",
			"Object color",
		   "Change object",
			  "Add object",
		   "Delete object",
		   "Select object",
			   "Send card"
	};
	
	int i, text_offset = -GP142_XMAX + 8, font_size = 12;
	int y, label_y;
	
	/* draw rectangles and labels for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		y = GP142_YMAX - i * BUTTON_HEIGHT;
		
		label_y = y - BUTTON_HEIGHT/2;

		/* recess button and highlight label if it was clicked last */
		if (i == last_button && last_sub_button == NO_SUB_BUTTON) {
			DrawBevelRect(-GP142_XMAX + 4,
						  y - 2,
						  -GP142_XMAX + BUTTON_WIDTH,
						  y - BUTTON_HEIGHT - 2,
						  RECESSED);
			GP142_printfXY(SELECTED_COLOR, text_offset, label_y,
						   font_size, b_labels[i]);
		} else {
			DrawBevelRect(-GP142_XMAX + 4,
						  y - 2,
						  -GP142_XMAX + BUTTON_WIDTH,
						  y - BUTTON_HEIGHT - 2,
						  RAISED);
			GP142_printfXY(BLACK, text_offset, label_y, font_size,
						   b_labels[i]);
		}

		/* draw the subordinate buttons if applicable */
		if (i == last_button &&
			(i == CHANGE_CARD_DESIGN || i == CHANGE_BACKGROUND_COLOR ||
			 i == CHANGE_OBJECT_COLOR || i == CHANGE_OBJECT_NUM ||
			 i == ADD_OBJECT) )
			DrawSubButtons(i, last_sub_button);

	} /* end for loop */
}


/* 
 * Draws the subordinate buttons for the selected button.
 */
void DrawSubButtons(int button_num, int last_sub_button)
{
/* To add buttons, change the #defined constant NUM_SUB_BUTTONS and
 * add messages to this array.
 */
	char dir_labels [NUM_SUB_BUTTONS][10] = {
			"Next",
		"Previous"
	};

	char add_labels [NUM_SUB_BUTTONS][10] = {
		 "Message",
		   "Shape"
	};

	int i, sub_text_offset = -GP142_XMAX + BUTTON_WIDTH + 4;
	int font_size = 12;
	int sub_y, sub_label_y;

	for (i = 0; i < NUM_SUB_BUTTONS; i++) {
		sub_y = GP142_YMAX - button_num * BUTTON_HEIGHT -
				i * SUB_BUTTON_HEIGHT;
		
		/* recess button if it was clicked last */
		if (i == last_sub_button) {
			DrawBevelRect(-GP142_XMAX + BUTTON_WIDTH,
						 sub_y - 2,
						 -GP142_XMAX + BUTTON_WIDTH + SUB_BUTTON_WIDTH,
						 sub_y - SUB_BUTTON_HEIGHT - 2,
						 RECESSED);
		} else {
			DrawBevelRect(-GP142_XMAX + BUTTON_WIDTH,
						 sub_y - 2,
						 -GP142_XMAX + BUTTON_WIDTH + SUB_BUTTON_WIDTH,
						 sub_y - SUB_BUTTON_HEIGHT - 2,
						 RAISED);
		}

		sub_label_y = sub_y - SUB_BUTTON_HEIGHT;

		/* Use next/previous or message/shape, as appropriate. */
		if (button_num == CHANGE_CARD_DESIGN ||
			button_num == CHANGE_BACKGROUND_COLOR ||
			button_num == CHANGE_OBJECT_COLOR ||
			button_num == CHANGE_OBJECT_NUM) {
		
			if (i == last_sub_button)
				GP142_printfXY(SELECTED_COLOR, sub_text_offset,
							   sub_label_y, font_size, dir_labels[i]);
			else
				GP142_printfXY(BLACK, sub_text_offset,
							   sub_label_y, font_size, dir_labels[i]);

		} else if (button_num == ADD_OBJECT) {

			if (i == last_sub_button)
				GP142_printfXY(SELECTED_COLOR, sub_text_offset,
							   sub_label_y, font_size, add_labels[i]);
			else
				GP142_printfXY(BLACK, sub_text_offset,
							   sub_label_y, font_size, add_labels[i]);
		}

	} /* end for loop */
}


/*
 * Draws a gray beveled rectangle bounded by the two corner points
 * given. The status parameter indicates whether to darken the upper
 * left or lower right lines, to give the appearance of raising or
 * recessing the rectangle.
 */
#define BEVEL_WIDTH	2
void DrawBevelRect(int x1, int y1, int x2, int y2, int status)
{
	int ul_color, lr_color;
	
	if (status == RECESSED) {
		ul_color = BLACK;
		lr_color = MED_GRAY;
	} else {
		ul_color = MED_GRAY;
		lr_color = BLACK;
	}

	/* Draw the rectangle as four separate lines. */
	GP142_lineXY(ul_color, x1, y1, x1, y2, BEVEL_WIDTH);
	GP142_lineXY(ul_color, x1, y1, x2, y1, BEVEL_WIDTH);
	GP142_lineXY(lr_color, x2, y1, x2, y2, BEVEL_WIDTH);
	GP142_lineXY(lr_color, x1, y2, x2, y2, BEVEL_WIDTH);

}


/*
 * Draw the card, and draw the envelope animation when sending card.
 *
 * When the animation is not enabled (*time < 0), this function simply
 * calls DrawCard. When the animation is activated, it draws the card
 * as well as the animation. DrawCard must be called from this function
 * instead of main because the animation draws both above and below
 * the card.
 */
#define ENV_BORDER_WIDTH		20
#define CARD_COVER_TIME			625
#define CLOSE_HALF_FLAP_TIME	800
#define CLOSE_FULL_FLAP_TIME	1000
#define MOVE_ENVELOPE_TIME		1400
#define TIME_INCREMENT			5
void DrawCardArea (int *time, int card_design, int background_color,
				   int status, int object_count,
				   const int object_type[], const int object_num[],
				   const int object_color[],
				   const int object_x[], const int object_y[])
{
	int rel_time;		/* Relative time for each part of animation. */

	if (*time == -1) {
	/* Simply draw the card if send animation is not activated. */

		DrawCard (card_design, background_color, status,
				  object_count, object_type, object_num,
				  object_color, object_x, object_y);

	} else if (*time < CARD_COVER_TIME) {
	/* If animation is activated, move envelope up from bottom of
	 * the window until the card is covered by it. */

		rel_time = *time;

		/* Draw open flap behind the card. */
		DrawFlapUp(rel_time);

		/* Draw the card. */
		DrawCard (card_design, background_color, status,
				  object_count, object_type, object_num,
				  object_color, object_x, object_y);

		/* Draw envelope over card. */
		DrawEnvelope(rel_time);

		*time += TIME_INCREMENT;
	
	} else if (*time < CLOSE_FULL_FLAP_TIME) {
	/* Once card is covered, close the flap. */

		rel_time = *time - CARD_COVER_TIME;

		DrawCard (card_design, background_color, status,
				  object_count, object_type, object_num,
				  object_color, object_x, object_y);

		DrawEnvelope(CARD_COVER_TIME);

		DrawFlapClosing(rel_time);

		*time += TIME_INCREMENT;

	} else if (*time < MOVE_ENVELOPE_TIME) {
	/* Once flap is closed, move the envelope. */

		rel_time = *time - CLOSE_FULL_FLAP_TIME;

		/* Display message behind the card. */
		GP142_printfXY(BLACK,
				PAGE_LLX + 85,
				PAGE_LLY + PAGE_HEIGHT - 135,
				60, "COME\nBACK\nSOON!");

		/* Draw the envelope and fully closed flap */
		DrawEnvelope(CARD_COVER_TIME + rel_time);

		DrawFlapDown(rel_time);

		*time += TIME_INCREMENT;

	} else {

		/* Draw the message */
		GP142_printfXY(BLACK,
			PAGE_LLX + 85,
			PAGE_LLY + PAGE_HEIGHT - 135,
			60, "COME\nBACK\nSOON!");

	}

}


/* Draws an open flap to fit the envelope, beginning at the bottom of
 * the screen and raised by the amount of time passed.
 */
void DrawFlapUp(int time)
{
	GP142_triangleXY(RED,
			PAGE_LLX,
			-GP142_YMAX - PAGE_HEIGHT/2 + time,
			PAGE_LLX + PAGE_WIDTH,
			-GP142_YMAX - PAGE_HEIGHT/2 + time,
			PAGE_LLX + PAGE_WIDTH/2,
			-GP142_YMAX + time,
			FILL);
	GP142_triangleXY(WHITE,
			PAGE_LLX + ENV_BORDER_WIDTH,
			-GP142_YMAX - PAGE_HEIGHT/2 + time,
			PAGE_LLX + PAGE_WIDTH - ENV_BORDER_WIDTH,
			-GP142_YMAX - PAGE_HEIGHT/2 + time,
			PAGE_LLX + PAGE_WIDTH/2,
			-GP142_YMAX - ENV_BORDER_WIDTH + time,
			FILL);
}


/* Draws a flap to fit the envelope fixed at the top of the card, and
 * folded down by the amount of time passed.
 */
void DrawFlapClosing(int time)
{
	/* This is the time it takes to close the flap halfway. */
	int offset = CLOSE_HALF_FLAP_TIME + ENV_BORDER_WIDTH -
					CARD_COVER_TIME;

	GP142_triangleXY(RED,
			PAGE_LLX,
			PAGE_LLY + PAGE_HEIGHT,
			PAGE_LLX + PAGE_WIDTH,
			PAGE_LLY + PAGE_HEIGHT,
			PAGE_LLX + PAGE_WIDTH/2,
			PAGE_LLY + PAGE_HEIGHT*3/2 - time,
			FILL);

	/* These change the way the white part of the flap is drawn,
	 * because it must look different when the flap is before half
	 * closed, half closed, and beyond half closed. */
	if (time < (offset - ENV_BORDER_WIDTH*2)) {
		
		GP142_triangleXY(WHITE,
				PAGE_LLX + ENV_BORDER_WIDTH,
				PAGE_LLY + PAGE_HEIGHT,
				PAGE_LLX + PAGE_WIDTH - ENV_BORDER_WIDTH,
				PAGE_LLY + PAGE_HEIGHT,
				PAGE_LLX + PAGE_WIDTH/2,
				PAGE_LLY + PAGE_HEIGHT*3/2 - ENV_BORDER_WIDTH -	time,
				FILL);
	
	} else if (time < offset) {

		/* Do nothing, because white part should not be visible. */

	} else {

		/* The value of (time - offset)/9 starts at zero and increases
		 * gradually to provide the illusion of perspective in the
		 * white part of the closing flap.
		 */
		GP142_triangleXY(WHITE,
				PAGE_LLX + ENV_BORDER_WIDTH*2 - (time - offset)/9,
				PAGE_LLY + PAGE_HEIGHT - (time - offset)/9,
				PAGE_LLX + PAGE_WIDTH - ENV_BORDER_WIDTH*2 +
					(time - offset)/9,
				PAGE_LLY + PAGE_HEIGHT - (time - offset)/9,
				PAGE_LLX + PAGE_WIDTH/2,
				PAGE_LLY + PAGE_HEIGHT*3/2 + ENV_BORDER_WIDTH -
					time,
				FILL);
	}

}


/* Draws a closed flap to fit the envelope, beginning at the top of
 * the card and raised by the amount of time passed.
 */
void DrawFlapDown(int time)
{
	GP142_triangleXY(RED,
			PAGE_LLX,
			PAGE_LLY + PAGE_HEIGHT + time,
			PAGE_LLX + PAGE_WIDTH,
			PAGE_LLY + PAGE_HEIGHT + time,
			PAGE_LLX + PAGE_WIDTH/2,
			PAGE_LLY + PAGE_HEIGHT/2 - ENV_BORDER_WIDTH + time,
			FILL);
	GP142_triangleXY(WHITE,
			PAGE_LLX + ENV_BORDER_WIDTH*2 - ENV_BORDER_WIDTH,
			PAGE_LLY + PAGE_HEIGHT - ENV_BORDER_WIDTH + time,
			PAGE_LLX + PAGE_WIDTH - ENV_BORDER_WIDTH*2 + ENV_BORDER_WIDTH,
			PAGE_LLY + PAGE_HEIGHT - ENV_BORDER_WIDTH + time,
			PAGE_LLX + PAGE_WIDTH/2,
			PAGE_LLY + PAGE_HEIGHT/2 + time,
			FILL);

}


/* Draws an envelope the size of the card, beginning at the bottom of
 * the screen and raised by the amount of time passed.
 */
void DrawEnvelope(int time)
{
	GP142_rectangleXY(RED,
			PAGE_LLX,
			-GP142_YMAX - PAGE_HEIGHT + time,
			PAGE_LLX + PAGE_WIDTH,
			-GP142_YMAX - PAGE_HEIGHT*3/2 + time,
			FILL);
	GP142_rectangleXY(WHITE,
			PAGE_LLX + ENV_BORDER_WIDTH,
			-GP142_YMAX - PAGE_HEIGHT + time,
			PAGE_LLX + PAGE_WIDTH - ENV_BORDER_WIDTH,
			-GP142_YMAX + ENV_BORDER_WIDTH - PAGE_HEIGHT*3/2 + time,
			FILL);
	GP142_triangleXY(RED,
			PAGE_LLX,
			-GP142_YMAX - PAGE_HEIGHT + time,
			PAGE_LLX,
			-GP142_YMAX - PAGE_HEIGHT/2 + time,
			PAGE_LLX + PAGE_WIDTH/2,
			-GP142_YMAX - PAGE_HEIGHT + time,
			FILL);
	GP142_triangleXY(RED,
			PAGE_LLX + PAGE_WIDTH,
			-GP142_YMAX - PAGE_HEIGHT + time,
			PAGE_LLX + PAGE_WIDTH,
			-GP142_YMAX - PAGE_HEIGHT/2 + time,
			PAGE_LLX + PAGE_WIDTH/2,
			-GP142_YMAX - PAGE_HEIGHT + time,
			FILL);
	GP142_triangleXY(WHITE,
			PAGE_LLX + ENV_BORDER_WIDTH,
			-GP142_YMAX - PAGE_HEIGHT + time,
			PAGE_LLX + ENV_BORDER_WIDTH,
			-GP142_YMAX - ENV_BORDER_WIDTH*2 - PAGE_HEIGHT/2 + time,
			PAGE_LLX + PAGE_WIDTH/2 - ENV_BORDER_WIDTH,
			-GP142_YMAX - PAGE_HEIGHT + time,
			FILL);
	GP142_triangleXY(WHITE,
			PAGE_LLX + PAGE_WIDTH - ENV_BORDER_WIDTH,
			-GP142_YMAX - PAGE_HEIGHT + time,
			PAGE_LLX + PAGE_WIDTH - ENV_BORDER_WIDTH,
			-GP142_YMAX - ENV_BORDER_WIDTH*2 - PAGE_HEIGHT/2 + time,
			PAGE_LLX + PAGE_WIDTH/2 + ENV_BORDER_WIDTH,
			-GP142_YMAX - PAGE_HEIGHT + time,
			FILL);

}


/*
 * Render (draw) the card and all its contents.
 */
void DrawCard (int card_design, int background_color, int status,
			   int object_count, const int object_type[],
			   const int object_num[], const int object_color[],
			   const int object_x[], const int object_y[])
{
	int font_size = 14;
	int i;
	
	/*
	 * Draw the background of the card.
	 */
	GP142_rectangleXY (background_color,
			PAGE_LLX, PAGE_LLY,
			PAGE_LLX+ PAGE_WIDTH, PAGE_LLY+ PAGE_HEIGHT,
			FILL);

	/*
	 * Draw the design of the card (if any).
	 */
	switch ((card_design - FIRST_DESIGN_NUM) % MAX_DESIGNS) {
	case NO_DESIGN:
		break;
	case DESIGN_SPRING:
		DrawSpringDesign();
		break;
	case DESIGN_CALENDAR:
		DrawCalendarDesign();
		break;
	case DESIGN_HEART:
		DrawHeartDesign();
		break;
	case DESIGN_BORDER:
		DrawBorderDesign();
		break;
	default:
		/* Unknown design selected */
		break;

	} /*end switch*/
	
	/* 
	 * Draw shapes on the card, if any currently exist.
	 */
	if (object_count > 0) {
		for (i = 0; i < MAX_OBJECTS; i++) {
			if (object_type[i] == OBJECT_SHAPE) {
				DrawShape (object_num[i], object_color[i],
						   object_x[i], object_y[i]);
			} else if (object_type[i] == OBJECT_MESSAGE) {
				DrawMessage (object_num[i], object_color[i],
							 object_x[i], object_y[i]);
			}
		}
	}
	
	/*
	 * Write an appropriate status message below the background
	 */
	switch (status) {
	case STATUS_ADD_SHAPE:
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on the card to add a shape");
		break;
	case STATUS_ADD_MESSAGE:
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on the card to add a message");
		break;
	case STATUS_SELECTED:
		GP142_printfXY (RED, PAGE_LLX, PAGE_LLY-40, font_size-3,
			"Object selected. Click on the card to move it, or click button to change color, etc.");
		break;
	case STATUS_SELECT_OBJECT:
		GP142_printfXY (BLUE, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on the card to select an object (click close to its origin)");
		break;
	case STATUS_NO_SELECT:
		GP142_printfXY (BLUE, PAGE_LLX, PAGE_LLY-40, font_size,
			"There are no objects to select");
		break;
	case STATUS_DELETE_OBJECT:
		GP142_printfXY (RED, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on currently selected object to confirm delete");
		break;
	case STATUS_NO_DELETE:
		GP142_printfXY (RED, PAGE_LLX, PAGE_LLY-40, font_size,
			"There are no objects to delete");
		break;
	case STATUS_MAX_OBJECTS:
		GP142_printfXY (RED, PAGE_LLX, PAGE_LLY-40, font_size,
			"You have already placed the maximum number of objects");
		break;
	case STATUS_SEND_CARD:
		/* No message when sending card. */
		break;
	default:
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Status error -- notify programmer");
		break;
	} /*end switch */
	
} /*end DrawCard */


/*
 * Draw a sunny spring day on the card.
 */
#define GROUND_HEIGHT	75
#define GRASS_HEIGHT	5
#define GRASS_SPACING	5
#define NUM_GRASS		(PAGE_WIDTH / GRASS_SPACING)
#define FLOWERS_RADIUS	5
#define FLOWERS_HEIGHT	30
#define FLOWERS_SPACING 50
#define FLOWERS_OFFSET	30
#define NUM_FLOWERS		(PAGE_WIDTH / FLOWERS_SPACING)
#define SUN_RADIUS		30
#define SUN_CENTER_X	PAGE_LLX + 300
#define SUN_CENTER_Y	PAGE_LLY + 300
void DrawSpringDesign (void) {

	int i;		/* loop counter */
	
	int r;		/* radius of flower center */
	int x, y;	/* coordinates of flower center */

	/* the ground */
	GP142_rectangleXY (GREEN,
		PAGE_LLX, PAGE_LLY,
		PAGE_LLX + PAGE_WIDTH, PAGE_LLY + GROUND_HEIGHT,
		FILL);

	/* the grass */
	for (i = 0; i < NUM_GRASS; i++) {
		GP142_lineXY (GREEN,
			PAGE_LLX + i * GRASS_SPACING, PAGE_LLY + GROUND_HEIGHT,
			PAGE_LLX + i * GRASS_SPACING,
			PAGE_LLY + GROUND_HEIGHT + GRASS_HEIGHT,
			NOFILL);
	}

	/* the flowers */
	r = FLOWERS_RADIUS;
	for (i = 0; i < NUM_FLOWERS; i++) {
		x = PAGE_LLX + i * FLOWERS_SPACING + FLOWERS_OFFSET;
		y = PAGE_LLY + GROUND_HEIGHT + FLOWERS_HEIGHT;
		GP142_lineXY (GREEN, x, y, x, y - FLOWERS_HEIGHT, NOFILL);
		GP142_ovalXY (RED, x - 3*r, y + r, x, y - r, FILL);
		GP142_ovalXY (RED, x, y + r, x + 3*r, y - r, FILL);
		GP142_ovalXY (RED, x - r, y + 3*r, x + r, y, FILL);
		GP142_ovalXY (RED, x - r, y - 3*r, x + r, y, FILL);
		GP142_circleXY (YELLOW, x, y, r);
	}

	/* the sun */
	GP142_circleXY (YELLOW, SUN_CENTER_X, SUN_CENTER_Y, SUN_RADIUS);
	
}


/*
 * Draw a Valentine's Day calendar.
 */
#define CALENDAR_X		PAGE_LLX + 25
#define CALENDAR_Y		PAGE_LLY + 150
#define CALENDAR_HEIGHT 180
#define CALENDAR_WIDTH	154
#define WEEKS	6
#define WEEK_HEIGHT		(CALENDAR_HEIGHT / WEEKS)
#define DAYS	7
#define DAY_WIDTH		(CALENDAR_WIDTH / DAYS)
void DrawCalendarDesign (void) {

	int font_size = 18;
	int x1, y1, x2, y2;		/* rectangle bound points */
	int i, j, day = 0;		/* counters */

	/* Draw the white calendar background. */
	x1 = CALENDAR_X;
	y1 = CALENDAR_Y;
	x2 = x1 + CALENDAR_WIDTH;
	y2 = y1 + CALENDAR_HEIGHT;

	GP142_rectangleXY (WHITE, x1, y1, x2, y2, FILL);

	/* Draw calendar outline and month name */
	GP142_rectangleXY (BLACK, x1, y1, x2, y2, NOFILL);
	GP142_printfXY (RED, x1 + 10, y2 + 3 - WEEK_HEIGHT, font_size + 2,
					"FEBRUARY  '99");

	/* Draw the days and weeks of the month as a series of a series of
	 * rectangles.
	 */
	for (i = 1; i < WEEKS; i++) {
		y1 = CALENDAR_Y + CALENDAR_HEIGHT - (i * WEEK_HEIGHT);
		y2 = y1 - WEEK_HEIGHT;

		for (j = 0; (j < DAYS && day < 29); j++) {
			x1 = CALENDAR_X + (j * DAY_WIDTH);
			x2 = x1 + DAY_WIDTH;
					
			GP142_rectangleXY (BLACK, x1, y1, x2, y2, NOFILL);

			/* Draw Sundays/holidays in red, others in black.
			 * Insert a space before one-digit days to center them.
			 * Cross out the incorrect Valentine's Day and circle the
			 * proper one.
			 */
			if (day == 14) {
				GP142_printfXY(RED, x1 + 3, y2 + 5, font_size,
							   "%d", day);
				GP142_lineXY(BLUE, x1, y1, x2, y2, 2);
				GP142_lineXY(BLUE, x1, y2, x2, y1, 2);
			} else if (day == 7) {
				GP142_printfXY(RED, x1 + 3, y2 + 5, font_size,
							   " %d", day);
			} else if (day == 15 || day == 21 || day == 28) {
				GP142_printfXY(RED, x1 + 3, y2 + 5, font_size,
							   "%d", day);
			} else if (day == 26) {
				GP142_printfXY(BLACK, x1 + 3, y2 + 5, font_size,
							   "%d", day);
				GP142_ovalXY(BLUE, x1, y1, x2, y2, 2);
			} else if (day > 9) {
				GP142_printfXY(BLACK, x1 + 3, y2 + 5, font_size,
							   "%d", day);
			} else if (day != 0) {
				GP142_printfXY(BLACK, x1 + 3, y2 + 5, font_size,
							   " %d", day);
			}

			day++;

		} /* end day loop */
	
	} /* end week loop */

}


/*
 * Draw a heart design the size of the card.
 */
#define HEART_DESIGN_WIDTH	200
void DrawHeartDesign (void) {

	GP142_ovalXY (RED,
			PAGE_LLX,
			PAGE_LLY + PAGE_HEIGHT,
			PAGE_LLX + HEART_DESIGN_WIDTH,
			PAGE_LLY + PAGE_HEIGHT - HEART_DESIGN_WIDTH,
			FILL);
	GP142_ovalXY (RED,
			PAGE_LLX + PAGE_WIDTH,
			PAGE_LLY + PAGE_HEIGHT,
			PAGE_LLX + PAGE_WIDTH - HEART_DESIGN_WIDTH,
			PAGE_LLY + PAGE_HEIGHT - HEART_DESIGN_WIDTH,
			FILL);
	GP142_triangleXY (RED,
			PAGE_LLX + PAGE_WIDTH/2 - HEART_DESIGN_WIDTH + 30,
			PAGE_LLY + PAGE_HEIGHT/2 + 30,
			PAGE_LLX + PAGE_WIDTH/2 + HEART_DESIGN_WIDTH - 30,
			PAGE_LLY + PAGE_HEIGHT/2 + 30,
			PAGE_LLX + PAGE_WIDTH/2,
			PAGE_LLY + PAGE_HEIGHT/2 - HEART_DESIGN_WIDTH + 30,
			FILL);

}


/*
 * Draws a red border over the edges of the card, with diagonal white
 * lines running back and forth within the border.
 */
#define BORDER_WIDTH		30
#define NUM_BORDER_LINES	12	/* must be even to work properly */
void DrawBorderDesign (void)
{
	int i;
	int h_line_length = PAGE_WIDTH / NUM_BORDER_LINES;
	int v_line_length = PAGE_HEIGHT / NUM_BORDER_LINES;

	/* Draw the red border around the card edges. */
	GP142_rectangleXY(RED,
			PAGE_LLX,
			PAGE_LLY,
			PAGE_LLX + PAGE_WIDTH,
			PAGE_LLY + BORDER_WIDTH,
			FILL);
	GP142_rectangleXY(RED,
			PAGE_LLX,
			PAGE_LLY,
			PAGE_LLX + BORDER_WIDTH,
			PAGE_LLY + PAGE_HEIGHT,
			FILL);
	GP142_rectangleXY(RED,
			PAGE_LLX + PAGE_WIDTH,
			PAGE_LLY,
			PAGE_LLX + PAGE_WIDTH - BORDER_WIDTH,
			PAGE_LLY + PAGE_HEIGHT,
			FILL);
	GP142_rectangleXY(RED,
			PAGE_LLX,
			PAGE_LLY + PAGE_HEIGHT,
			PAGE_LLX + PAGE_WIDTH,
			PAGE_LLY + PAGE_HEIGHT - BORDER_WIDTH,
			FILL);

	/* Draw white lines through the border. */
	for (i = 0; i < NUM_BORDER_LINES; i += 2) {
		GP142_lineXY(WHITE,
			PAGE_LLX,
			PAGE_LLY + i * v_line_length,
			PAGE_LLX + BORDER_WIDTH,
			PAGE_LLY + (i + 1) * v_line_length,
			NOFILL);
		GP142_lineXY(WHITE,
			PAGE_LLX + BORDER_WIDTH,
			PAGE_LLY + (i + 1) * v_line_length,
			PAGE_LLX,
			PAGE_LLY + (i + 2) * v_line_length,
			NOFILL);
		GP142_lineXY(WHITE,
			PAGE_LLX + PAGE_WIDTH,
			PAGE_LLY + i * v_line_length,
			PAGE_LLX + PAGE_WIDTH - BORDER_WIDTH,
			PAGE_LLY + (i + 1) * v_line_length,
			NOFILL);
		GP142_lineXY(WHITE,
			PAGE_LLX + PAGE_WIDTH - BORDER_WIDTH,
			PAGE_LLY + (i + 1) * v_line_length,
			PAGE_LLX + PAGE_WIDTH,
			PAGE_LLY + (i + 2) * v_line_length,
			NOFILL);

		GP142_lineXY(WHITE,
			PAGE_LLX + i * h_line_length,
			PAGE_LLY,
			PAGE_LLX + (i + 1) * h_line_length,
			PAGE_LLY + BORDER_WIDTH,
			NOFILL);
		GP142_lineXY(WHITE,
			PAGE_LLX + (i + 1) * h_line_length,
			PAGE_LLY + BORDER_WIDTH,
			PAGE_LLX + (i + 2) * h_line_length,
			PAGE_LLY,
			NOFILL);
		GP142_lineXY(WHITE,
			PAGE_LLX + i * h_line_length,
			PAGE_LLY + PAGE_HEIGHT,
			PAGE_LLX + (i + 1) * h_line_length,
			PAGE_LLY + PAGE_HEIGHT - BORDER_WIDTH,
			NOFILL);
		GP142_lineXY(WHITE,
			PAGE_LLX + (i + 1) * h_line_length,
			PAGE_LLY + PAGE_HEIGHT - BORDER_WIDTH,
			PAGE_LLX + (i + 2) * h_line_length,
			PAGE_LLY + PAGE_HEIGHT,
			NOFILL);

	}

}


/*
 * Draw one shape.
 */
void DrawShape(int shape_type, int color, int bottom_x, int bottom_y) {

	switch ((shape_type - FIRST_SHAPE_NUM) % MAX_SHAPES) {
	case SHAPE_HEART:
		DrawHeart (bottom_x, bottom_y, color);
		break;
	case SHAPE_STAR:
		DrawStar (bottom_x, bottom_y, color);
		break;
	case SHAPE_CIRCLE:
		DrawCircle (bottom_x, bottom_y, color);
		break;
	case SHAPE_DIAMOND:
		DrawDiamond (bottom_x, bottom_y, color);
		break;
	case SHAPE_OVAL:
		DrawOval (bottom_x, bottom_y, color);
		break;
	case SHAPE_FLOWER:
		DrawFlower (bottom_x, bottom_y, color);
		break;
	default:
		/* Unknown shape selected */
		break;

	} /*end switch*/

}


/*
 * Draw a heart on the card. A heart will be made by two overlapping
 * circles and a triangle.
 *
 * The pulsation is caused by adding a small, random amount to the 
 * heart height each time this function is called.
 */
#define HEART_WIDTH 35
void DrawHeart (int x, int y, int color) {

	int h = (HEART_WIDTH/2) + (rand () %4);

	GP142_circleXY (color, x - h + 5, y, h);
	GP142_circleXY (color, x + h - 5, y, h);
	GP142_triangleXY (color, x- h*2 +5, y-5,
						x + h*2 -5, y-5,
						x, y - h*2 -5, FILL);

}


/*
 * Draw a star on the card.
 *
 * We make the star twinkle by selecting the number of points randomly.
 * Each time it is redrawn, it is slightly different than the time
 * before, which makes it appear to pulsate,
 */
#define STARPOINTS 5
#define STARRADIUS 40
#define PI 3.1416
void DrawStar (int x, int y, int color) {

	int sp;
	double angle;
	int x1, y1;

	sp = STARPOINTS + (rand () %3);
	for (angle = 0.0; angle < 2 * PI; angle = angle + (2*PI)/sp) {
		x1 = (int) (cos (angle) * STARRADIUS);
		y1 = (int) (sin (angle) * STARRADIUS);
		GP142_lineXY (color, x+x1, y+y1, x-x1, y-y1, 1);

	}
}


/*
 * Draw a circle on the card.
 *
 * The pulsation is caused by adding a small, random amount to the
 * circle radius each time this function is called.
 */
#define CIRCLERADIUS 20
void DrawCircle (int x, int y, int color) {

	int r = CIRCLERADIUS + (rand () %4);

	GP142_circleXY (color, x, y, r);

}


/*
 * Draw a diamond on the card.
 *
 * The pulsation is caused by adding a small, random amount to the
 * diamond width each time this function is called.
 */
#define DIAMOND_WIDTH 15
void DrawDiamond (int x, int y, int color) {

	int w = DIAMOND_WIDTH + (rand () %4);

	GP142_triangleXY (color, x - w, y,
					  x + w, y,
					  x, y + 2*w, FILL);
	GP142_triangleXY (color, x - w, y,
					  x + w, y,
					  x, y - 2*w, FILL);

}


/*
 * Draw a set of ovals on the card.
 *
 * The pulsation is caused by subtracting a small, random amount from
 * the oval height each time this function is called.
 *
 * The number of ovals varies somewhat randomly, with a fall-through
 * switch statement designed so that the smallest oval is always
 * present, and the largest is most rarely present.
 */
#define OVAL_HEIGHT 40
void DrawOval (int x, int y, int color) {

	int h1 = OVAL_HEIGHT - (rand () %4);
	int h2 = (int) (h1 / 1.3);
	int h3 = (int) (h1 / 2.0);
	int h4 = (int) (h1 / 4.0);

	switch (rand () %4) {
	case 0:
		GP142_ovalXY (color, x - h1, y + h1/2,
					  x + h1, y - h1/2, NOFILL);
	case 1:
		GP142_ovalXY (color, x - h3, y + h3/2,
					  x + h3, y - h3/2, NOFILL);
	case 2:
		GP142_ovalXY (color, x - h2, y + h2/2,
					  x + h2, y - h2/2, NOFILL);
	case 3:
		GP142_ovalXY (color, x - h4, y + h4/2,
					  x + h4, y - h4/2, NOFILL);
	}

}


/*
 * Draw a flower on the card. A flower will be made by four overlapping
 * ovals and a circle.
 *
 * The pulsation is caused by adding a small, random amount to the 
 * radius each time this function is called.
 */
#define FLOWER_RADIUS 10
void DrawFlower (int x, int y, int color) {

	int r = FLOWER_RADIUS + (rand () %4);

	GP142_ovalXY (color, x - 3*r, y + r,
				  x, y - r, FILL);
	GP142_ovalXY (color, x, y + r,
				  x + 3*r, y - r, FILL);
	GP142_ovalXY (color, x - r, y + 3*r,
				  x + r, y, FILL);
	GP142_ovalXY (color, x - r, y - 3*r,
				  x + r, y, FILL);
	GP142_circleXY (color + 1, x, y, r);

}


/*
 * Draw one message.
 *
 * The pulsation is caused by adding a small, random amount to the
 * font size each time this function is called.
 * 
 */
void DrawMessage(int msg_type, int color, int bottom_x, int bottom_y) {

	int font_size = 12;
	int fs = font_size + (rand () %6);

	switch ((msg_type - FIRST_MESSAGE_NUM) % MAX_MESSAGES) {
	case MESSAGE_1:
		GP142_printfXY (color, bottom_x, bottom_y, fs,
						"B MINE");
		break;
	case MESSAGE_2:
		GP142_printfXY (color, bottom_x, bottom_y, fs,
						"TRUE LUV");
		break;
	case MESSAGE_3:
		GP142_printfXY (color, bottom_x, bottom_y, fs,
						"Be My Valentine");
		break;
	case MESSAGE_4:
		GP142_printfXY (color, bottom_x, bottom_y, fs,
						"Sweetie Pie");
		break;
	case MESSAGE_5:
		GP142_printfXY (color, bottom_x, bottom_y, fs,
						"LUV U");
		break;
	default:
		/* Unknown message selected */
		break;

	} /*end switch*/

}


/*
 * Mouse clicks occur in one of three regions of interest: on buttons, 
 * on the card, or anywhere else.
 */
int  ClassifyMouseClick(int mouse_x, int mouse_y)
{
  if ((mouse_x < -GP142_XMAX + BUTTON_WIDTH + SUB_BUTTON_WIDTH) &&
      (mouse_y < GP142_YMAX) &&
      (mouse_y > (GP142_YMAX - NUM_BUTTONS*BUTTON_HEIGHT)))
    return CLICK_BUTTON;
  else if (mouse_x >= PAGE_LLX && mouse_x < PAGE_LLX + PAGE_WIDTH
	  && mouse_y >= PAGE_LLY && mouse_y < PAGE_LLY + PAGE_HEIGHT)
    return CLICK_CARD;
  else
    return CLICK_OUTSIDE;

}


/*
 * Decide where a mouse click occured, and react appropriately.
 */
void HandleMouseClick(int mouse_x, int mouse_y, int *action_time,
					  int *card_design, int *background_color,
					  int *last_button, int *last_sub_button,
					  int *status, int *current_object,
					  int *object_count, int object_type[],
					  int object_num[], int object_color[],
					  int object_x[], int object_y[])
{

	int mouse_click;
	mouse_click = ClassifyMouseClick(mouse_x, mouse_y);

	if (*status == STATUS_SEND_CARD) {
		/* Ignore mouse clicks when sending card. */
	} else if (mouse_click == CLICK_BUTTON) {
		/* A button has been clicked.  Call a function to handle it*/
		/* most of the variables passed as args will NOT change*/
		HandleButton(mouse_x, mouse_y, action_time, card_design,
					 background_color, last_button, last_sub_button,
					 status, *current_object, *object_count,
					 object_type, object_color, object_num);

	} else if (mouse_click == CLICK_CARD) {
		/* The card has been clicked. Call a function to handle it*/
		HandleObject(mouse_x, mouse_y, status, last_button,
					 last_sub_button, current_object, object_count,
					 object_type, object_num, object_color,
					 object_x, object_y);

	} else if (mouse_click == CLICK_OUTSIDE) {
		/* A blank area has been clicked. Deselect all buttons*/
		*last_button = NO_BUTTON;
		*last_sub_button = NO_SUB_BUTTON;
		if (*object_count > 0)
			*status = STATUS_SELECTED;
		else
			*status = STATUS_ADD_SHAPE;
	}
	
}

/* 
 * Figure out which button was clicked, if any, and do the right 
 * thing.
 */
void HandleButton(int x, int y, int *action_time, int *card_design,
				  int *background_color, int *last_button,
				  int *last_sub_button, int *status,
				  int current_object, int object_count,
				  const int object_type[], int object_color[],
				  int object_num[])
{
	int buttonNum = NO_BUTTON;
	int subButtonNum = NO_SUB_BUTTON;
	
	/* If we're outside of this area, we could not have clicked on
	 * a button.
	 */
	if (x > -GP142_XMAX + BUTTON_WIDTH + SUB_BUTTON_WIDTH)
		return ;	/*That's an error situation*/
	
	/* We know the mouse clicked in a column of buttons. Calculate
	 * which column, and then which button.
	 */
	if (x < -GP142_XMAX + BUTTON_WIDTH) {
	/* Clicked in the main button column. */

		buttonNum = (GP142_YMAX - y)/BUTTON_HEIGHT;
		subButtonNum = NO_SUB_BUTTON;

	} else if (*last_button == (GP142_YMAX - y)/BUTTON_HEIGHT) {
	/* Clicked the the subordinate button column. */

		buttonNum = (GP142_YMAX - y)/BUTTON_HEIGHT;
		subButtonNum = (GP142_YMAX - buttonNum * BUTTON_HEIGHT - y) /
						SUB_BUTTON_HEIGHT;

	}

	*last_button = buttonNum;
	*last_sub_button = subButtonNum;
	if (object_count > 0)
		*status = STATUS_SELECTED;

	switch(buttonNum) {
	case NO_BUTTON:
		/* No button was clicked, so do nothing.
		 * (This means the click was in the button area, but not where
		 * any buttons are currently present) */
		break;
	case CHANGE_CARD_DESIGN:
		/* Change the card design */
		ChangeCardDesign(card_design, subButtonNum);
		break;
	case CHANGE_BACKGROUND_COLOR:
		/* Change the background color */
		ChangeBackgroundColor(background_color, subButtonNum);
		break;
	case CHANGE_OBJECT_COLOR:
		/* Change the color of the currently selected object */
		if (current_object >= 0)
			ChangeObjectColor(&object_color[current_object],
							  subButtonNum);
		break;
	case CHANGE_OBJECT_NUM:
		/* Change the type of the currently selected object */
		if (current_object >= 0)
			ChangeObjectNum(&object_num[current_object],
							object_type[current_object], subButtonNum);
		break;
	case ADD_OBJECT:
		/* Change status to add message or shape mode. */
		if (subButtonNum == MESSAGE)
			*status = STATUS_ADD_MESSAGE;
		else if (subButtonNum == SHAPE)
			*status = STATUS_ADD_SHAPE;
		break;
	case DELETE_OBJECT:
		/* Change status to delete object mode. */
		if (subButtonNum == NO_SUB_BUTTON) {
			if (current_object >= 0)
				*status = STATUS_DELETE_OBJECT;
			else
				*status = STATUS_NO_DELETE;
		}
		break;
	case SELECT_OBJECT:
		/* Select an object on the card */
		if (subButtonNum == NO_SUB_BUTTON) {
			if (current_object >= 0)
				*status = STATUS_SELECT_OBJECT;
			else
				*status = STATUS_NO_SELECT;
		}
		break;
	case SEND_CARD:
		/* Change to send card mode, and start time counter. */
		*status = STATUS_SEND_CARD;
		*action_time = 0;
		break;
	default:
		*status = 999;
		/*Bad news if this is ever reached! */
		break;
	}
}


/* Changes the card design in the given direction. */
void ChangeCardDesign(int *card_design, int direction)
{
	if (direction == NEXT)
		*card_design = (*card_design + 1) % MAX_DESIGNS;
	else if (direction == PREVIOUS) {
		*card_design = (*card_design - 1);
		if (*card_design < 0)
			*card_design = *card_design + MAX_DESIGNS;
	}
}


/* Changes the background color in the given direction. */
void ChangeBackgroundColor(int *background_color, int direction)
{
	if (direction == NEXT)
		*background_color = (*background_color + 1) % MAX_COLORS;
	else if (direction == PREVIOUS) {
		*background_color = (*background_color - 1);
		if (*background_color < 0)
			*background_color = *background_color + MAX_COLORS;
	}
}


/* Changes the selected object's color in the given direction. */
void ChangeObjectColor(int *object_color, int direction)
{
	if (direction == NEXT)
		*object_color =	(*object_color + 1) % MAX_COLORS;
	else if (direction == PREVIOUS) {
		*object_color = (*object_color - 1);
		if (*object_color < 0)
			*object_color = *object_color + MAX_COLORS;
	}
}


/* Changes the number of the selected object in the given direction. */
void ChangeObjectNum(int *object_num, int object_type, int direction)
{
	int max;

	if (object_type == OBJECT_SHAPE)
		max = MAX_SHAPES;
	else if (object_type == OBJECT_MESSAGE)
		max = MAX_MESSAGES;

	if (direction == NEXT)
		*object_num = (*object_num + 1) % max;
	else if (direction == PREVIOUS) {
		*object_num = (*object_num - 1);
		if (*object_num < 0)
			*object_num = *object_num + max;
	}
}


/* 
 * Figure out under what circumstances the card was clicked, and do
 * the right thing.
 */
void HandleObject(int mouse_x, int mouse_y, int *status,
				  int *last_button, int *last_sub_button,
				  int *current_object, int *object_count,
				  int object_type[], int object_num[],
				  int object_color[], int object_x[], int object_y[])
{
	switch (*status) {
	case STATUS_ADD_SHAPE:
		AddShape(mouse_x, mouse_y, status, current_object,
				 object_count, &object_type[*object_count],
				 &object_num[*object_count],
				 &object_color[*object_count],
				 &object_x[*object_count], &object_y[*object_count]);
		break;
	case STATUS_ADD_MESSAGE:
		AddMessage(mouse_x, mouse_y, status, current_object,
				   object_count, &object_type[*object_count],
				   &object_num[*object_count],
				   &object_color[*object_count],
				   &object_x[*object_count], &object_y[*object_count]);
		break;
	case STATUS_SELECTED:
		ObjectMove(mouse_x, mouse_y, &object_x[*current_object],
				  &object_y[*current_object]);
		break;
	case STATUS_SELECT_OBJECT:
		ObjectSelect(mouse_x, mouse_y, status, current_object,
					 *object_count, object_x, object_y);
		break;
	case STATUS_DELETE_OBJECT:
		ObjectDelete(mouse_x, mouse_y, status, current_object,
					 object_count, object_type, object_num,
					 object_color, object_x, object_y);
		break;
	case STATUS_NO_DELETE:
	case STATUS_NO_SELECT:
		*status = STATUS_ADD_SHAPE;
		break;
	case STATUS_MAX_OBJECTS:
		*status = STATUS_SELECTED;
		break;
	default:
		*status = 999;
		/*Bad news if this is ever reached! */
	}

	/* Deselect buttons after clicking an object */
	if (*status == STATUS_SELECTED || *object_count == 0) {
		*last_button = NO_BUTTON;
		*last_sub_button = NO_SUB_BUTTON;
	}

}
				  
					  
/* If there's room for another shape, add it to the card. */
void AddShape(int x, int y, int *status, int *current_object,
			  int *object_count, int *object_type, int *object_num,
			  int *object_color, int *object_x, int *object_y)
{
	if (*object_count < MAX_OBJECTS) {
		*status = STATUS_SELECTED;
		*current_object = *object_count;
		*object_count += 1;
		*object_type = OBJECT_SHAPE;
		*object_num = SHAPE_HEART;
		*object_color = BLACK;
		*object_x = x;
		*object_y = y;
	} else
		*status = STATUS_MAX_OBJECTS;

}


/* If there's room for another message, add it to the card. */
void AddMessage(int x, int y, int *status, int *current_object,
				int *object_count, int *object_type, int *object_num,
				int *object_color, int *object_x, int *object_y)
{
	if (*object_count < MAX_OBJECTS) {
		*status = STATUS_SELECTED;
		*current_object = *object_count;
		*object_count += 1;
		*object_type = OBJECT_MESSAGE;
		*object_num = MESSAGE_1;
		*object_color = BLACK;
		*object_x = x;
		*object_y = y;
	} else
		*status = STATUS_MAX_OBJECTS;

}


/* Move the selected object to the point clicked by the mouse. */
void ObjectMove(int x, int y, int *object_x, int *object_y)
{
	*object_x = x;
	*object_y = y;

}


/* Selects the object clicked on by the mouse. */
#define TOLERANCE 5
void ObjectSelect(int x, int y, int *status, int *current_object,
				  int object_count,
				  const int object_x[], const int object_y[])
{
	int i;

	for (i = 0; i < object_count; i++) {
		/* Check if click was within range of tolerance of object's
		 * coordinates. */
		if ( (x + TOLERANCE) > object_x[i] && 
			 (x - TOLERANCE) < object_x[i] &&
			 (y + TOLERANCE) > object_y[i] &&
			 (y - TOLERANCE) < object_y[i] ) {
			
			*current_object = i;
			*status = STATUS_SELECTED;			
		}
	}
}


/* Deletes the selected object when it is clicked on. */
void ObjectDelete(int x, int y, int *status, int *current_object,
				  int *object_count, int object_type[],
				  int object_num[], int object_color[],
				  int object_x[], int object_y[])
{
	int i, object = *current_object;

	ObjectSelect(x, y, status, current_object, *object_count,
					 object_x, object_y);
	
	if (object == *current_object && *status == STATUS_SELECTED) {
		/* Shift down by one all objects beyond the deleted object. */
		for (i = *current_object; i < (*object_count - 1); i++) {
			object_type[i] = object_type[i + 1];
			object_num[i] = object_num[i + 1];
			object_color[i] = object_color[i + 1];
			object_x[i] = object_x[i + 1];
			object_y[i] = object_y[i + 1];
		}
		while (i < MAX_OBJECTS) {
			object_type[i] = OBJECT_NONE;
			i++;
		}

		*object_count -= 1;
		*current_object = *object_count - 1;
		if (*object_count == 0)
			*status = STATUS_ADD_SHAPE;
	}

}
